
/* Leuchtfeuer für große Leuchttürme        by J.Reinert   für Arduino Nano V3
 *  
 * letzte Änderung 18.05.2019 10:00 
 * 
 * Was wäre ein Deko-Leuchtturm im Garten ohne rotierendes Licht. Es gibt Leuchttürme 
 * aus Holz, die etwa 135cm hoch und 50cm breit sind. Leider nur mit einfacher 230V-
 * Handleuchte im Kopf oder einer Solar-Lichterkette. 
 * 
 * Ich habe nun, ohne Mechanik, ein 'drehendes' Leuchtfeuer mit adressierbaren LEDs 
 * (WS2812b) gebastelt. Diese Software ist ein einfaches Lauflicht für kreisförmig 
 * angeordnete Pixel-LEDs 
 * 
 * Hardware-Anschluss am Arduino: 
 *   +5V    Power +5Volt 
 *   GND    Ground
 *   D13    Datenausgang WS2812b für LEDs
 * 
 * Konstanten im Programm:
 * L_WIDTH      Anzahl LEDs in ganzer Breite 
 * L_HEIGHT     Anzahl LEDs in ganzer Höhe
 * L_COLOR      Farbe des Leuchtfeuers
 * F_WIDTH      Breite des Leuchtfeuers in Pixel
 * F_BRIGHTNES  Helligkeit des Feuers
 * SPEED_DELAY  Drehgeschwindigkeit. Je kleiner der Wert, um so schneller wird der Effekt
 * DATAPIN      Arduino-Pin, wo die LEDs angeschlossen sind
 *
 * Einzubindende Bibliothek:  Adafruit_NeoPixel
 * 
 */


#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
  #include <avr/power.h>
#endif

// --------------------------------------------------------------------------------------
// Hardwareabhängige Werte. Bitte anpassen!

#define L_WIDTH 26          // Anzahl Pixel-LEDs in der Breite (umlaufend)
#define L_HEIGHT 3          // Anzahl Pixel-LEDs in der Höhe
#define L_COLOR 0xFFFFFF    // Farbe des Lichtes
#define F_WIDTH 3           // Breite der Flamme in Pixel
#define F_BRIGHTNES 255     // Helligkeit des Leuchtfeuers (0 bis 255)       
#define SPEED_DELAY 140     // Je kleiner der Wert, um so schneller fliegt der Ball
#define DATAPIN 13          // Daten-Ausgang zu Pixel-LEDs

// ---------------------------------------------------------------------------------------

//  Library-Parameter setzen
Adafruit_NeoPixel strip = Adafruit_NeoPixel(L_WIDTH*L_HEIGHT, DATAPIN, NEO_RGB + NEO_KHZ800);

/* --------------------------------------------------------------------------
   Programm-Setup
   --------------------------------------------------------------------------
 */
void setup() {
  strip.begin();                      // Alle LEDs ausschalten und Datenausgabe starten
  strip.show();                       // Speicherinhalt in LED-Streifen übertragen
  strip.setBrightness(F_BRIGHTNES);   // in voller Helligkeit
}

/* --------------------------------------------------------------------------
   Hauptprogramm-Schleife
   --------------------------------------------------------------------------
 */ 
void loop() {
  for(int j=0;j<L_WIDTH; j++) {                     // Alle Pixel der Reihe nach adressieren
    ScrollPixel();                                  // Alle Pixel in Matrix verschieben
    if (j<F_WIDTH) {                                // So lange Feuer-Breite nicht erreicht...
      for(int i=0;i<L_HEIGHT;i++) {                 // ... erstes Pixel jeder Zeile...
        strip.setPixelColor(i*L_WIDTH,L_COLOR);     // ... einschalten
      }
    }
    strip.show();                                   // LED in LED-Streifen übertragen
    delay(SPEED_DELAY);                             // kurz warten
  }   
}

/* --------------------------------------------------------------------------
 * Alle LEDs eine Position nach oben schieben
 * --------------------------------------------------------------------------
 */
void ScrollPixel() {
  for(int j=L_WIDTH-1; j>0; j--) {                  // Alle Pixel horizontal bearbeiten
    for(int i=0;i<L_WIDTH;i++) {                    // Alle Pixel vertikal bearbeiten
      strip.setPixelColor(j+i*L_WIDTH,strip.getPixelColor(j+(i*L_WIDTH)-1));  // Alle Pixel eine Position nach oben 
    }
  }
  for(int i=0;i<L_WIDTH;i++) {                      // Alle Anfangspixel löschen
    strip.setPixelColor(i*L_WIDTH,0);  
  }
}


